require 'rubygame'
require 'lib/popup'
require 'lib/goods'
require 'lib/day'
require 'lib/announce'

class City
  attr_reader :image, :rect, :produces, :consumes, :prices
  attr_accessor :name
  include Rubygame::Sprites::Sprite
  
  @@font = 0
  
  @info = false
  @@popup = nil
  
  def initialize(name,bg)
    super()
    
    @produces= []
    @consumes = []
    
    init_market
    
    @@popup ||= Popup.instance
    
    self.name = name
    @@font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",12) if @@font == 0
    label = @@font.render(name,true,[20,20,20])

    @image = Rubygame::Surface.new([label.size[0],label.size[1]+10])
    @image.fill bg
    @image.set_colorkey(bg)
        
    dot = Rubygame::Surface[ (rand < 0.7 ? "small_dot.png" : "large_dot.png") ]
    dot.blit(@image, [@image.size[0]/2-4,2])
    label.blit(@image,[0,10])
    @rect = @image.make_rect
    
    @distance_to = {}
  end

  def init_market
    Announce.instance.announce("Prices in #{self.name} have changed!",nil,5) if not @prices.nil?
    @prices = {}
    Goods.raw.each { |good| @prices[good] = 20 + (25 * rand) }
    Goods.refined.each { |good| @prices[good] = 40 + (35 * rand) }
    Day.add_task(Day.day + 3 + (rand * 10).to_i,Proc.new{self.init_market})
  end
    
  def real_x
    @rect.x + @rect.w/2
  end
  
  def real_y
    @rect.y + 6
  end
  
  def info=(f)
    @info=f
  end
  
  def toggle_info
    @info = !@info
    if @@popup.city == self
      @@popup.visible = @info
    else
      @@popup.city.info=false if not @@popup.city.nil?
      @@popup.city = self
      @@popup.visible = true
    end
  end
  
  def world=(array)
    array.each do |city|
      if city != self
        distance = ( (@rect.x - city.rect.x) ** 2 + (@rect.y - city.rect.y) ** 2) ** 0.5
        @distance_to[city] = distance
      end
    end
  end
  
  def distance_to(city)
    @distance_to[city]
  end
  
  def nearest(n)
    set = []
    @distance_to.each_pair { |city,dist| set.push [dist,city] }
    set.sort! { |a,b| a[0] <=> b[0] }
    return set[0..n].map { |pair| pair[1] }
  end
  
  def draw(surf)
    r = super(surf)
    return r
  end
  
  def update
  
  end
  
  alias size rect
end
