require 'rubygame'
require 'lib/visible'
require 'singleton'

class HelpOverlay
  include Rubygame::Sprites::Sprite
  include Singleton
  include Visible
  
  attr_reader :image,:rect
  
  def initialize
    super
    
    @text = ["To get started, click the company logo in the upper-right corner and buy your",
             "first truck. Once you've done that, designate the truck's starting location by",
             "clicking on a city and selecting a starting cargo. Set up a shipping route by",
             "selecting additional cities and goods to buy and sell. Stops on a route don't",
             "have to be directly connected -- your drivers are smart enough to find a route!",
             " ",
             "  Prices differ wildly between cities, so one of your first goals should be to",
             "find a route with guaranteed profit. Deliver a cargo produced by one city to",
             "another where demand is higher to increase your profits. Click on a city to see",
             "which goods it produces and which are in demand.",
             " ",
             "  The ticker in the upper-left shows how much cash you have on hand. Try to keep",
             "it in the green, boss.",
             " ",
             "Happy trucking!"]
    
    font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",24)
    hfont = Rubygame::TTF.new("res/fonts/FreeSans.ttf",36)
    @header = hfont.render("Welcome to Interstate Trucking!",true,[0,0,0])
    @labels = @text.map { |string| font.render(string,true,[10,10,10]) }
    @width = 0
    @labels.each { |l| @width = l.size[0] if l.size[0] > @width }
    
    @settings = Settings.instance
    self.visible=false
    @screen = nil
    @alpha = 0
  end
  
  def draw(surf)
    @screen ||= surf
    
    if visible?
      surf.draw_box_s([0,0],[@settings[:width],@settings[:height]],[255,255,255,@alpha])
      
      @header.blit(surf,[@settings[:width]/2-@header.size[0]/2,5])
      @labels.each_with_index { |label,i| label.blit(surf,[@settings[:width]/2-@width/2,60+i*label.size[1]+2]) }
    end
  end
  
  def hide
    super
    draw(@screen)
  end
  
  def show
    super
    @alpha = 0
  end
  
  def update
    if @alpha < 200 and visible?
      @alpha += 20
    end
  end
end
  
