require 'rubygame'
require 'lib/city'
require 'lib/goods'
require 'singleton'

class Map
  attr_reader :image, :rect, :cities, :linked
  include Singleton
  include Rubygame::Sprites::Sprite
  
  Width = 1024
  Height = 640
  
  def initialize
    super
    @image = Rubygame::Surface.new([Width,Height])
    bg = [235,230,225]
    @image.fill bg
    @rect = @image.make_rect
    
    font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",10)
    font2 = Rubygame::TTF.new("res/fonts/FreeSans.ttf",8)
    font.bold=true
    
    city_list = []
    File.open("res/data/cities.list","r").each_line { |l| city_list.push l.strip }
    num_cities = 26 + (rand * 12).to_i
    @cities = []
    num_cities.times { @cities.push City.new(city_list.delete_at((rand*city_list.size).to_i), bg) }
    
    gx = -50
    gy = -50
    
    @cities.each do |city|
      city.rect.x = (rand * 120).to_i + gx
      city.rect.y = (rand * 230).to_i + gy
      gx += 165
      if gx >= 1100
        gx = 0
        gy += 245
      end
      
      city.produces.concat Goods.raw.reject { |x| rand > 0.5 }
      city.produces.concat Goods.refined.reject { |x| rand > 0.8 }
      
      city.consumes.concat Goods.raw - city.produces
      city.consumes.concat(Goods.refined - city.produces).reject { |x| rand > 0.9 }
    end
    
    @highways = []
    @linked = []
    @fast_roads = []
    @cities.each do |city|
      city.world = @cities
      
      nearest = city.nearest((rand*1).to_i + 1)
      nearest.each { |other|
        if not @linked.include?([city,other]) and not @linked.include?([other,city])
          number = (rand*899).to_i+100
          number_t = font.render("#{number}",true,[20,20,20])
          @highways.push [ [city.rect.x+city.rect.width/2,city.rect.y+6],[other.rect.x+other.rect.width/2,other.rect.y+6], number, number_t ]
          @linked.push [city,other]
          @linked.push [other,city]
          @fast_roads.push [city,other]
          @fast_roads.push [other,city]
        end
      }
    end
    
    @roads = []
    @cities.each do |city|
      city.world = @cities
      
      nearest = city.nearest((rand*2).to_i + 4)
      nearest.each { |other|
        if not @linked.include?([city,other]) and not @linked.include?([other,city])
          number = (rand*89).to_i+10
          number_t = font2.render("#{number}",true,[20,20,20])
          @roads.push [ [city.rect.x+city.rect.width/2,city.rect.y+6],[other.rect.x+other.rect.width/2,other.rect.y+6], number, number_t ]
          @linked.push [city,other]
          @linked.push [other,city]
        end
      }
    end
    
    @linked.uniq!
    
    @highway = Rubygame::Surface['highway.png']
  end
  
  def highways
    @fast_roads
  end
  
  def find_path(start,goal)
    path = {}
    closed_set = []
    g_score = {}
    f_score = {}
    open_set = [start]
    g_score[start] = 0
    while not open_set.empty?
      x = open_set.shift
      if x == goal
        return reconstruct(path,goal)
      end
      closed_set.push x
      @cities.reject { |city| not @linked.include?([x,city]) }.each do |neighbor|
        if not closed_set.include?(neighbor)
          temp_g = g_score[x] + x.distance_to(neighbor)
          better = false
          if not open_set.include?(neighbor)
            open_set.push neighbor
            better = true
          elsif temp_g < g_score[neighbor]
            better = true
          end
          if better
            path[neighbor] = x
            g_score[neighbor] = temp_g
            f_score[neighbor] = g_score[neighbor] + (neighbor.distance_to(goal) || 0)
          end
        end
      end
    end
  end
  
  def reconstruct(path,node)
    if not path[node].nil?
      p = reconstruct(path,path[node])
      return [p,node].flatten
    else
      return []
    end
  end
  
  def fix_route(route)
    new_route = []
    route.each_with_index do |city,i|
      i = -1 if i >= route.size-1
      new_route.push city if not new_route[new_route.size-1] == city
      
      if @linked.include?([city,route[i+1]]) or @linked.include?([route[i+1],city])
        new_route.push route[i+1]
      else
        new_route.concat find_path(city,route[i+1])
      end
    end
    new_route.pop if new_route[0] == new_route[new_route.size-1]
    return new_route
  end
  
  def draw(screen)
    r = super
    @roads.each { |line| 
      screen.draw_line_a(line[0],line[1],[170,170,170])
      x = (line[0][0]+line[1][0])/2 - @highway.size[0]/2
      y = (line[0][1]+line[1][1])/2 - @highway.size[1]/2
      
      screen.draw_box_s([x-5,y+2],[x+line[3].size[0]+5,y+line[3].size[1]-3],[255,255,255])
      screen.draw_box([x-5,y+2],[x+line[3].size[0]+5,y+line[3].size[1]-3],[60,60,60])
      line[3].blit(screen,[x,y])
    }
    @highways.each { |line| 
      dim = ( (line[0][0] - line[1][0]).abs > (line[0][1] - line[1][1]).abs ? 1 : 0)
      line[0][dim]-= 1
      line[1][dim]-= 1
      screen.draw_line_a(line[0],line[1],[242,191,36])
      line[0][dim]+= 1
      line[1][dim]+= 1
      screen.draw_line_a(line[0],line[1],[242,211,66])
      line[0][dim]+= 1
      line[1][dim]+= 1
      screen.draw_line_a(line[0],line[1],[242,191,36])
      line[0][dim]-= 1
      line[1][dim]-= 1
      
      x = (line[0][0]+line[1][0])/2 - @highway.size[0]/2
      y = (line[0][1]+line[1][1])/2 - @highway.size[1]/2
      
      @highway.blit(screen,[x,y])
      line[3].blit(screen,[x+5,y+3])
    }
    return r
  end
  
  def update
  
  end
end
