require 'rubygame'
require 'singleton'

class Popup
  include Rubygame::Sprites::Sprite
  include Singleton
  
  attr_writer :visible
  attr_reader :city, :rect, :image
  
  def initialize
    super
    
    @image = Rubygame::Surface["popup.png"]
    @rect = @image.make_rect
    @city = nil
    @header = nil
    @font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",16)
    @tinyfont = Rubygame::TTF.new("res/fonts/FreeSans.ttf",14)
    @produce = []
    @consume = []
    @visible = false
  end
  
  def visible?
    @visible
  end
  
  def city=(city)
    @city = city
    @header = @font.render(city.name,true,[220,220,220])
    @produce = ["   Sells:",city.produces].flatten.map { |item| 
      surface = Rubygame::Surface.new(@tinyfont.size_text("#{item} $00.00"))
      surface.fill([141,138,135])
      surface.set_colorkey([141,138,135])
      label = @tinyfont.render(item,true,[250,250,250])
      label.blit(surface,[0,0])
      if city.prices[item]
        price = @tinyfont.render("$#{city.prices[item].to_i}",true,[255,160,160])
        price.blit(surface,[label.size[0]+2,0])
      end
      surface
    }
    @consume = ["   Wants:",city.consumes].flatten.map { |item| 
      surface = Rubygame::Surface.new(@tinyfont.size_text("#{item} $00.00"))
      surface.fill([141,138,135])
      surface.set_colorkey([141,138,135])
      label = @tinyfont.render(item,true,[250,250,250])
      label.blit(surface,[0,0])
      if city.prices[item]
        price = @tinyfont.render("$#{city.prices[item].to_i}",true,[180,255,160])
        price.blit(surface,[label.size[0]+2,0])
      end
      surface
    }
  end
  
  def update
  
  end
  
  def visible=(f)
    @visible = f
  end

  def draw(surf)
    if visible?
      x = @city.rect.x + @city.rect.width/2
      y = @city.rect.y + 6
      x -= 200 if x > 700
      y -= 200 if y > 500
      self.rect.x = x
      self.rect.y = y
      r = super(surf)
      @header.blit(surf,[x+100-@header.size[0]/2,y+3])
      surf.draw_line([x+10,y+@header.size[1]+1],
                     [x-13+self.rect.w,y+@header.size[1]+1],
      [160,160,160])
      
      @produce.each_with_index { |s,i| s.blit(surf,[x+10,y + @header.size[1]+3+i*15]) }
      @consume.each_with_index { |s,i| s.blit(surf,[x+100,y + @header.size[1]+3+i*15]) }
      return r
    end
    return Rubygame::Rect.new(0,0,0,0)
  end
end
